@@ -1,13 +1,14 @@
 import logging
 from contextlib import asynccontextmanager
-from typing import AsyncIterator, Union
+from typing import AsyncGenerator, AsyncIterator, Type, TypeVar, Union
 
-from aioch import Client
-from clickhouse_driver import errors
+import asynch
+from asynch.cursors import Cursor, DictCursor
+from asynch.errors import ClickHouseException
 
 from tesseract_olap.backend import Backend
 from tesseract_olap.backend.exceptions import UpstreamInternalError
-from tesseract_olap.common import AnyTuple
+from tesseract_olap.common import AnyDict
 from tesseract_olap.query import DataQuery, MembersQuery
 from tesseract_olap.query.exceptions import InvalidQuery
 from tesseract_olap.schema import Schema
@@ -16,6 +17,8 @@ from .sqlbuild import dataquery_sql, membersquery_sql
 
 logger = logging.getLogger("tesseract_olap.backend.clickhouse")
 
+CursorType = TypeVar("CursorType", bound=Cursor)
+
 
 class ClickhouseBackend(Backend):
     """Clickhouse Backend class
@@ -29,6 +32,7 @@ class ClickhouseBackend(Backend):
     """
 
     connection_string: str
+    pool: Union[asynch.pool.Pool, None]
 
     def __init__(self, connection_string: str) -> None:
         self.connection_string = connection_string
@@ -36,30 +40,38 @@ class ClickhouseBackend(Backend):
     def __repr__(self) -> str:
         return f"ClickhouseBackend('{self.connection_string}')"
 
-    @asynccontextmanager
-    async def _acquire(self):
-        client = Client.from_url(self.connection_string)
-        try:
-            yield client
-        except errors.ServerException as exc:
-            raise UpstreamInternalError(str(exc)) from None
-        finally:
-            await client.disconnect()
+    async def connect(self, **kwargs) -> asynch.pool.Pool:
+        pool = getattr(self, "pool", None)
+        if pool is None:
+            pool = await asynch.create_pool(dsn=self.connection_string)
+        self.pool = pool
+        return pool
 
-    async def connect(self, **kwargs):
-        pass
+    @asynccontextmanager
+    async def acquire(
+        self, curcls: Type[CursorType] = Cursor
+    ) -> AsyncGenerator[CursorType, None]:
+        pool = await self.connect()
+        try:
+            async with pool.acquire() as conn:
+                async with conn.cursor(cursor=curcls) as cursor:
+                    yield cursor
+        except ClickHouseException as exc:
+            raise UpstreamInternalError(str(exc)) from None
 
     def close(self):
-        pass
+        pool = getattr(self, "pool", None)
+        if pool is not None:
+            pool.close()
 
     async def wait_closed(self):
-        pass
+        pool = getattr(self, "pool", None)
+        if pool is not None:
+            await pool.wait_closed()
 
     async def execute(
-        self,
-        query: Union["DataQuery", "MembersQuery"],
-        **kwargs
-    ) -> AsyncIterator[AnyTuple]:
+        self, query: Union["DataQuery", "MembersQuery"], **kwargs
+    ) -> AsyncIterator[AnyDict]:
         """
         Processes the requests in a :class:`DataQuery` or :class:`MembersQuery`
         instance, sends the query to the database, and returns an `AsyncIterator`
@@ -79,19 +91,20 @@ class ClickhouseBackend(Backend):
                 "ClickhouseBackend only supports DataQuery and MembersQuery instances"
             )
 
-        async with self._acquire() as cursor:
-            result = await cursor.execute_iter(query=sql_builder.get_sql(),
-                                               params=sql_params,
-                                               settings={"max_block_size": 5000},
-                                               with_column_types=True)
-            async for row in result:
+        async with self.acquire(DictCursor) as cursor:
+            # AsyncIterator must be fully consumed before returning because
+            # async context closes connection prematurely
+            await cursor.execute(query=sql_builder.get_sql(), args=sql_params)
+            result = cursor.fetchall()
+            for row in result:
                 yield row
 
     async def ping(self) -> bool:
         """Checks if the current connection is working correctly."""
-        async with self._acquire() as cursor:
-            result = await cursor.execute("SELECT 1")
-            return result == [(1,)]
+        async with self.acquire() as cursor:
+            await cursor.execute("SELECT 1")
+            result = cursor.fetchone()
+            return result == (1,)
 
     async def validate_schema(self, schema: "Schema") -> None:
         """Checks all the tables and columns referenced in the schema exist in
