@@ -7,11 +7,11 @@ purpose is organize and easily obtain the data needed for later steps.
 
 from collections import defaultdict
 from dataclasses import dataclass, field
-from typing import (Any, Dict, List, Mapping, Optional, Sequence, Set, Tuple,
-                    Union)
+from typing import List, Mapping, Optional, Set, Tuple, Union
 
 import immutables as immu
 
+from tesseract_olap.common import AnyDict
 from tesseract_olap.schema import (CubeTraverser, DimensionTraverser,
                                    HierarchyTraverser, LevelTraverser,
                                    SchemaTraverser)
@@ -64,7 +64,7 @@ class DataQuery:
             sorting=request.sorting,
         )
 
-    def get_sources(self) -> Sequence[Dict[str, Any]]:
+    def get_sources(self) -> Tuple[AnyDict, ...]:
         """Returns a sequence containing the sources for each measure in the Query.
 
         Only measures with a properly declared source will appear in this dict.
@@ -217,8 +217,8 @@ def _get_data_measurefields(cube: "CubeTraverser", req: "DataRequest"):
         fltr = req.filters.get(measure_name)
         if fltr is None:
             pass
-        elif len(fltr.condition) == 2: # is NumericConstraint
-            kwargs["constraint1"] = fltr.condition
+        elif len(fltr.condition) == 1: # is (NumConstr)
+            kwargs["constraint1"] = fltr.condition[0]
         elif len(fltr.condition) == 3: # is (NumConstr, LogicOp, NumConstr)
             kwargs["constraint1"] = fltr.condition[0]
             kwargs["joint"] = LogicOperator.from_str(fltr.condition[1])
