@@ -1,6 +1,6 @@
 import logging
 from pathlib import Path
-from typing import Union
+from typing import Union, overload
 
 import httpx
 from typing_extensions import Literal
@@ -8,6 +8,7 @@ from typing_extensions import Literal
 from tesseract_olap.backend import Backend
 from tesseract_olap.query import (DataQuery, DataRequest, DataResult,
                                   MembersQuery, MembersRequest, MembersResult)
+from tesseract_olap.query.exceptions import InvalidQuery
 from tesseract_olap.schema import Schema, SchemaTraverser, parse_xml_schema
 
 from .exceptions import UnknownBackendError, UnknownSchemaError
@@ -65,25 +66,40 @@ class OlapServer:
         except IndexError:
             return False
 
-    async def get_data(self, request: DataRequest, **kwargs) -> DataResult:
-        """Executes a DataRequest from the user to obtain aggregated data from
-        the backend. The response is wrapped in a :class:`DataResult` object.
+    @overload
+    async def execute(self, request: DataRequest, **kwargs) -> DataResult:
+        ...
+    @overload
+    async def execute(self, request: MembersRequest, **kwargs) -> MembersResult:
+        ...
+    async def execute(
+        self,
+        request: Union[DataRequest, MembersRequest],
+        **kwargs
+    ) -> Union[DataResult, MembersResult]:
         """
-        query = DataQuery.from_request(self.schema, request)
-        data = self.backend.get_data(query, **kwargs)
-        return DataResult(data=data,
-                          sources=query.get_sources(),
-                          query=request)
-
-    async def get_members(self, request: MembersRequest, **kwargs) -> MembersResult:
-        """Executes a MembersRequest from the user to obtain a list of the
-        different categories associated to the request, along with other
-        properties. The response is wrapped in a :class:`MembersResult` object.
+        If the request is an instance of :class:`DataRequest`, gets the aggregated
+        data from the backend and wraps it in a :class:`DataResult` object.
+        If the request is an instance of :class:`MembersRequest`, gets the list
+        of categories associated to the requested level and wraps the result in
+        a :class:`MembersResult` object.
         """
-        query = MembersQuery.from_request(self.schema, request)
-        data = self.backend.get_members(query, **kwargs)
-        return MembersResult(data=data,
-                             query=request)
+        if isinstance(request, MembersRequest):
+            query = MembersQuery.from_request(self.schema, request)
+            return MembersResult(
+                data=self.backend.execute(query, **kwargs),
+                query=request,
+            )
+        if isinstance(request, DataRequest):
+            query = DataQuery.from_request(self.schema, request)
+            return DataResult(
+                data=self.backend.execute(query, **kwargs),
+                sources=query.get_sources(),
+                query=request,
+            )
+        raise InvalidQuery(
+            "OlapServer only accepts instances of DataRequest or MembersRequest"
+        )
 
 
 def _setup_backend(connection: str):
